% Binary Tree Record Addition
%  Description:  This subroutine adds a record to the binary tree
%  Variables:  
%      records(1) = struct('start',y0,'finish',y,'sensitivity',A,...
%        'b_tree_height',integer,'accessed',integer);
%      b_tree(1) = struct('vv',vector,'aa',scalar,'lt',integer,'gt',integer,'leaf',integer);

function [] = isat_add(y0,y,A,i_branch,atol)

% ISAT Global Variables
global records n_records b_tree n_b_tree

n = size(y0,1);
atol_sq = atol' * atol;

% Index to new and old leafs
new = n_records + 1;
old = b_tree(i_branch).leaf;

% Changes to "records"

% Find the height of the new record
height = records(old).b_tree_height + 1;

% Add 1 to the height of the previous record
records(old).b_tree_height = height;

% Calculate the EOA
[U,S,V] = svd(A);
sigmas = diag(S);
for i = 1:n,
   sigmas(i) = max(atol_sq/2,sigmas(i));
end
modified_sensitivity = U * diag(sigmas) * V';

% Apply scaling matrix (B)
%new_eoa = modified_sensitivity' * B' * B * modified_sensitivity;
% The variables should already be scaled from the start
new_eoa = modified_sensitivity' * modified_sensitivity;

% Create a new record
records(new) = struct('start',y0,'finish',y,'sensitivity',A,...
   'eoa',new_eoa,'b_tree_height',height,'accessed',0);


% Changes to 'b_tree'  -------------------------------------------------------------------------------------------

% Step 1: Change leaf node into a branch node
%  -Define cutting plane (v' and a)
v = (records(new).start - records(old).start)';

sum_y1_y2 = (records(new).start + records(old).start);

a = 0.5 * (v * sum_y1_y2);

%  -Point to lt and gt
%  -Change previous b_tree leaf to a branch b_tree(i_branch) = 0;
b_tree(i_branch) = struct('vv',v,'aa',a,'lt',n_b_tree + 1,'gt',n_b_tree + 2,'leaf',0);

% Step 2: Grow 2 leaves
%  -Grow the old one
b_tree(n_b_tree+1) = struct('vv',zeros(1,n),'aa',0,'lt',0,'gt',0,'leaf',old);
%  -Grow the new one
b_tree(n_b_tree+2) = struct('vv',zeros(1,n),'aa',0,'lt',0,'gt',0,'leaf',new);

% Adding a leaf creates an additional record
n_records = n_records + 1;
% Adding a leaf creates an additional branch and leaf
n_b_tree = n_b_tree + 2;