% Binary Tree Search
%  Description:  This subroutine searches the binary tree for a close record.  The tree consists of 
%    branches and leaves at the ends of the branches.  The branch information is stored in "b_tree".  The
%    leaf information is stored in 'records'.  This algorithm receives "y0", a vector, and returns "i_branch1", a
%    pointer to the close leaf in "b_tree".  The variable "i_branch2" is a second close record.  In the event that
%    only one record exists, -1 is returned through "i_branch2".
%  Method:  Cutting plane is defined by  vv'*y0 = aa

function [i_branch1, i_branch2] = isat_bt_search(y0)

global n_records b_tree

% When there is only one record
if n_records == 1,
   i_branch1 = 1;
   i_branch2 = -1;
   return
end

% Start with the top record
i_branch1 = 1;

while b_tree(i_branch1).leaf == 0,
   % Keep track of the previous branch (i_branch2) and direction chosen (direction)
   i_branch2 = i_branch1;
   if b_tree(i_branch1).vv * y0 < b_tree(i_branch1).aa
      i_branch1 = b_tree(i_branch1).lt;
      direction = 'lt';
   else
      i_branch1 = b_tree(i_branch1).gt;
      direction = 'gt';
   end
end

% Start with the branch (or leaf) directly opposite the first leaf chosen
if direction == 'gt',
   i_branch2 = b_tree(i_branch2).lt;
else
   i_branch2 = b_tree(i_branch2).gt;
end

while b_tree(i_branch2).leaf == 0,
   if b_tree(i_branch2).vv * y0 < b_tree(i_branch2).aa
      i_branch2 = b_tree(i_branch2).lt;
   else
      i_branch2 = b_tree(i_branch2).gt;
   end
end

