% Grow Binary Tree Leaf

function [] = isat_growth(new_pt,i_leaf,atol)

global records

% Number of ISAT variables
n = size(new_pt,1);

% Compute the absolute tolerance squared
atol_sq = atol'*atol;

% Original EOA
A = records(i_leaf).eoa;

% Perform a Schur Decomposition of A
[Q,S] = schur(A);
e_vals = diag(S);

% Calculate Similarity Transform to Map Hyper-Ellipsoid to the a Unit Hyper-Sphere
T1 = diag(sqrt(e_vals/atol_sq)) * Q';

% Transform External Point into the Unit Hyper-Sphere Coordinates
T1_new_pt = T1 * new_pt;

% Determine the Magnitude and Direction of the Transformed External Point
T1_new_pt_mag = norm(T1_new_pt,2);
T1_new_pt_dir = T1_new_pt / T1_new_pt_mag;

% Generate an Orthonormal Basis with the Transformed External Point Direction as one
%   of the Column Vectors (it is the one that corresponds to the zero eigenvalue)
E = eye(n) - T1_new_pt_dir * T1_new_pt_dir';
[E_vecs,E_vals] = eig(E);

% Rotate the Unit Hyper-Sphere Coordinates to Align with the External Point Transformed Vector
T2 = inv(E_vecs);

% The Axis to be Expanded is the one that Corresponds to the Zero Eigenvalue
% This Routine Just Selects the Eigenvalue Position that is Closest to Zero
vec = abs(diag(E_vals));
smallest_eval = 1.0;
% Bandaid for a Bug:
% The code crashed when E was close to singular in working precision
% This resulted in no selection of variable "expand" and therefore a problem a few lines down
% One solution is to not grow the EOA but just live with the extra cost of integration
expand = -1;
for i = 1:n,
   if vec(i) < smallest_eval,
      expand = i;
      smallest_eval = vec(i);
   end
end

if expand == -1,
   % I should also probably print an error message here
   return
end

% Minimum Volume Expansion of the Unit Hyper-Sphere to Include the External Point
M2 = eye(n);
M2(expand,expand) = 1/T1_new_pt_mag^2;
   
% Transform the Ellipse Back to the Original Coordinates
A_expanded = atol_sq * (T1'*T2'*M2*T2*T1);

% Store the Expanded Ellipse
records(i_leaf).eoa = A_expanded;