% ISAT Initialization Routine

function [] = isat_initialize(y0,atol)

%clear global records
%clear global n_records 
%clear global b_tree
%clear global n_b_tree

% ISAT Global Variables
global records n_records b_tree n_b_tree

n = size(y0,1);
atol_sq = atol'*atol;

% Integrate to find the final states
y = isat_integrate(y0);   
% Calculate Sensitivity Matrix
A = isat_sensitivities(y0);


% Initialize Variables
%  For Structure "records"
%	  start = vector of initial states at time = t0
%    finish = vector of final states at time = t0 + dt
%    sensitivity = sensitivity matrix A(i,j) = (dy(i) / dy0(j))
%    b_tree_height = height in the binary tree - used by post-processing function 'tree_analysis.m'
%    accessed = number of times the leaf is accessed
records = struct('start',zeros(n,1),'finish',zeros(n,1),'sensitivity',zeros(n,n),...
   'eoa',zeros(n,n),'b_tree_height',0,'accessed',0);
%  For Structure 'b_tree'
%    vv' = transpose of the vector defining the cutting plane
%    aa = scalar as part of the definition of the cutting plane =>  vv'*y0 = aa
%    lt = pointer to the next node if v'*y0 < a
%    gt = pointer to the next node if v'*y0 >= a
%    leaf = pointer to the index of leaves in Structure "records"
%           this value is equal to zero if the node is another branch
b_tree = struct('vv',zeros(1,n),'aa',0,'lt',0,'gt',0,'leaf',0);

% Calculate the EOA
[U,S,V] = svd(A);
sigmas = diag(S);
for i = 1:n,
   sigmas(i) = max(atol_sq/2,sigmas(i));
end
modified_sensitivity = U * diag(sigmas) * V';
% Apply scaling matrix (B)
%new_eoa = modified_sensitivity' * B' * B * modified_sensitivity;
% The variables should already be scaled from the start
new_eoa = modified_sensitivity' * modified_sensitivity;

% Create the first record
records(1) = struct('start',y0,'finish',y,'sensitivity',A,'eoa',new_eoa,'b_tree_height',1,'accessed',0);
b_tree(1) = struct('vv',zeros(1,n),'aa',0,'lt',0,'gt',0,'leaf',1);

n_records = 1;
n_b_tree = 1;
