% Integrate the model to find y

function Ty = isat_integrate(Ty0)

global Tu T invT

% Options
model_name = 'cstr1';
model_type = 'ODE';
tspan = [0 0.2];
tol_integrator = 1e-6;

% The first element of y0 is not really a state but a manipulated variable in the model
Tu = Ty0(1,1);
Tx0 = Ty0(2:size(Ty0,1),1);

% Set the options for the integrator
if model_type == 'ODE',
   opts = odeset('AbsTol',tol_integrator,'RelTol',tol_integrator);
end
if model_type == 'DAE'
   opts = odeset('AbsTol',tol_integrator,'RelTol',tol_integrator,'Mass','M','MassSingular','yes');
end

% Integrate with ode15s
[t,Txs] = ode15s(model_name,tspan,Tx0,opts);

% Extract the final state values
Tx = Txs(size(Txs,1),:)';

% Return a vector
Ty = [Tu; Tx];