% Compute the sensitivities 

function A = isat_sensitivities(Ty0)

global Tu

% Options
model_name = 'cstr1';
model_name_x = [model_name, '_x'];
model_name_u = [model_name, '_u'];
model_type = 'ODE';
tspan = [0 0.2];
tol_integrator = 1e-6;

% The first element of y0 is not really a state but a step-wise constant
% manipulated variable in the model
Tu = Ty0(1,1);
Tx0 = Ty0(2:size(Ty0,1),1);

% Set the options for the integrator
if model_type == 'ODE',
   opts = odeset('AbsTol',tol_integrator,'RelTol',tol_integrator);
end
if model_type == 'DAE'
   opts = odeset('AbsTol',tol_integrator,'RelTol',tol_integrator,'Mass','M','MassSingular','yes');
end

% Sensitivity to Parameters (manipulated variables)
p = Tu;
[t,Tx,Su_all] = ode15s_sens(model_name_u,tspan,Tx0,opts,p,[],'initial_sensitivities_u');
last_Su_all = size(Su_all,1);
for i = 1:size(Tu,1)
   for j = 1:size(Tx0,1)
		Su(j,i) = Su_all(last_Su_all,j,i); 
	end
end

% Sensitivity to Initial Conditions (states)
p = Tx0;
[t,Tx,Sx_all] = ode15s_sens(model_name_x,tspan,Tx0,opts,p,[],'initial_sensitivities_x');
last_Sx_all = size(Sx_all,1);
for i = 1:size(Tx0,1)
	for j = 1:size(Tx0,1)
		Sx(j,i) = Sx_all(last_Sx_all,j,i); 
	end
end

% Return the sensitivity matrix
% The sensitivity of constant parameters is 1 for itself and 0 otherwise  
A = [eye(size(Tu,1),size(Ty0,1)); Su, Sx];