% Step test for sample calculations

% Step test for Model 1 - Single CSTR
% Created by John Hedengren

clear all
close all
% Global variables for ISAT testing
global errors

errors(1,:) = [0 0 0];


% Global variables for the simulation
global Tu T invT

% This is an optional statement - I'm only putting it here so I can have access to the ISAT database
%   after the simulation finishes
% ISAT Global Variables
global records n_records b_tree n_b_tree

figure(1);
hold on;
xlabel('Dimensionless Temperature')
ylabel('Dimensionless Concentration')
%axis([0.88 1.12 0.92 1.04]);

% Steady State Initial Conditions for the States
Ca_ss = 0.87725294608097;
T_ss = 324.475443431599;
x_ss = [Ca_ss;T_ss];

% Steady State Initial Condition for the Control
u_ss = 290;

% All Inputs and States
y_ss = [u_ss; x_ss];

T = diag(1./y_ss);
invT = diag(y_ss);

Ty_ss = T * y_ss;

% ISAT error tolerance
atol = 5e-3 * ones(size(T,1),1);
tf = 0.2;

time_steps = 10;

% Initial Conditions - Step Change in Tu
Tu = Ty_ss(1:size(u_ss,1),1) * 0.95;
Tx0 = Ty_ss(size(u_ss,1)+1:size(y_ss,1),1);

Tx0 = [1 0; 0 1] * Tx0;

Ty0 = [Tu;Tx0];

% Variables for plotting each case
c1 = 0;
c2 = 0;
c3 = 0;

for i = 1:time_steps,
   % Call ISAT   
   % dt and other options are specified in isat_integrate and isat_sensitivities
   [Ty_isat, A_isat, scenario] = isat(Ty0,atol);
   Ty_isat
   [t_ode15s,Tx_ode15s] = ode15s('cstr1',[0 tf],Tx0);
   
   Ty_ode15s = [Tu; Tx_ode15s(size(Tx_ode15s,1),:)'];
   
   total_error = sum(abs(Ty_isat - Ty_ode15s));
   
   switch scenario
   	case 1,
      	c1 = c1 + 1;
         x1(c1) = i;
         e1(c1) = total_error;
      case 2,
         c2 = c2 + 1;
         x2(c2) = i;
         e2(c2) = total_error;   
      case 3,
         c3 = c3 + 1;
         x3(c3) = i;
         e3(c3) = total_error;
   end
   
   if 1, 
      % Step test
      Tx0 = Tx_ode15s(size(Tx_ode15s,1),:)';
      %Tu=(Tu+1)/2;
      %'Tu'
      %Tu
      Ty0 = [Tu;Tx0];
   end
   
   if 0,
      % For testing purposes
      Tx0 = [0.99; 0.99];
      Ty0 = [Tu;Tx0];
   end
   
   if 0,      
      % Randomly set initial conditions
      mean = 1.0;
      var = 0.2;
      Tx0 = [-(rand(1)-0.5)*var+mean;-(rand(1)-0.5)*var+mean];
      Ty0 = [Tu;Tx0];
   end
   
   % Sidharth, you can adjust this pause function to speed up the
   % simulation
   pause(1)
   
end

figure(2);
hold on;
marker_size = 30;
% Number of Legend Entries
n_legend = 0;

if c1 >=1,
   scatter(x1,e1,marker_size,'k','c','filled');
   n_legend = n_legend + 1;
   legend_strings{n_legend} = 'retrieval';
end

if c2 >=1,
   scatter(x2,e2,marker_size,'b','^','filled');
   n_legend = n_legend + 1;
   legend_strings{n_legend} = 'growth';
end

if c3 >=1,
   scatter(x3,e3,marker_size,'g','s','filled');
   n_legend = n_legend + 1;
	legend_strings{n_legend} = 'addition';   
end

%sum_atol = sum(atol);
%plot([0 time_steps], [sum_atol sum_atol]);
%n_legend = n_legend + 1;
%legend_strings{n_legend} = 'limit';

legend(legend_strings);
xlabel('Integration Number');
ylabel('Sum of Errors');